@@ -12,18 +12,25 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from decorator import decorator
+import contextlib
 import inspect
+import netifaces
 import time
 
-import netifaces
+from decorator import decorator
+from neutronclient.common import exceptions as neutron_exc
 
 from shade import _log
 from shade import exc
+from shade import meta
+
 
 log = _log.setup_logging(__name__)
 
 
+_decorated_methods = []
+
+
 def _iterate_timeout(timeout, message, wait=2):
     """Iterate and raise an exception on timeout.
 
@@ -33,6 +40,13 @@ def _iterate_timeout(timeout, message, wait=2):
 
     """
 
+    try:
+        wait = float(wait)
+    except ValueError:
+        raise exc.OpenStackCloudException(
+            "Wait value must be an int or float value. {wait} given"
+            " instead".format(wait=wait))
+
     start = time.time()
     count = 0
     while (timeout is None) or (time.time() < start + timeout):
@@ -145,7 +159,7 @@ def normalize_keystone_services(services):
             'service_type': service_type,
         }
         ret.append(new_service)
-    return ret
+    return meta.obj_list_to_dict(ret)
 
 
 def normalize_nova_secgroups(groups):
@@ -159,11 +173,12 @@ def normalize_nova_secgroups(groups):
 
     :returns: A list of normalized dicts.
     """
-    return [{'id': g['id'],
-             'name': g['name'],
-             'description': g['description'],
-             'security_group_rules': normalize_nova_secgroup_rules(g['rules'])
-             } for g in groups]
+    ret = [{'id': g['id'],
+            'name': g['name'],
+            'description': g['description'],
+            'security_group_rules': normalize_nova_secgroup_rules(g['rules'])
+            } for g in groups]
+    return meta.obj_list_to_dict(ret)
 
 
 def normalize_nova_secgroup_rules(rules):
@@ -176,17 +191,18 @@ def normalize_nova_secgroup_rules(rules):
 
     :returns: A list of normalized dicts.
     """
-    return [{'id': r['id'],
-             'direction': 'ingress',
-             'ethertype': 'IPv4',
-             'port_range_min':
-                 None if r['from_port'] == -1 else r['from_port'],
-             'port_range_max':
-                 None if r['to_port'] == -1 else r['to_port'],
-             'protocol': r['ip_protocol'],
-             'remote_ip_prefix': r['ip_range'].get('cidr', None),
-             'security_group_id': r['parent_group_id']
-             } for r in rules]
+    ret = [{'id': r['id'],
+            'direction': 'ingress',
+            'ethertype': 'IPv4',
+            'port_range_min':
+                None if r['from_port'] == -1 else r['from_port'],
+            'port_range_max':
+                None if r['to_port'] == -1 else r['to_port'],
+            'protocol': r['ip_protocol'],
+            'remote_ip_prefix': r['ip_range'].get('cidr', None),
+            'security_group_id': r['parent_group_id']
+            } for r in rules]
+    return meta.obj_list_to_dict(ret)
 
 
 def normalize_nova_floating_ips(ips):
@@ -217,7 +233,7 @@ def normalize_nova_floating_ips(ips):
         ]
 
     """
-    return [dict(
+    ret = [dict(
         id=ip['id'],
         fixed_ip_address=ip.get('fixed_ip'),
         floating_ip_address=ip['ip'],
@@ -227,6 +243,7 @@ def normalize_nova_floating_ips(ips):
         status='ACTIVE'  # In neutrons terms, Nova floating IPs are always
                          # ACTIVE
     ) for ip in ips]
+    return meta.obj_list_to_dict(ret)
 
 
 def normalize_neutron_floating_ips(ips):
@@ -258,7 +275,7 @@ def normalize_neutron_floating_ips(ips):
         ]
 
     """
-    return [dict(
+    ret = [dict(
         id=ip['id'],
         fixed_ip_address=ip.get('fixed_ip_address'),
         floating_ip_address=ip['floating_ip_address'],
@@ -267,6 +284,7 @@ def normalize_neutron_floating_ips(ips):
                   ip.get('port_id') != ''),
         status=ip['status']
     ) for ip in ips]
+    return meta.obj_list_to_dict(ret)
 
 
 def localhost_supports_ipv6():
@@ -281,7 +299,7 @@ def localhost_supports_ipv6():
 
 
 def normalize_users(users):
-    return [
+    ret = [
         dict(
             id=user.get('id'),
             email=user.get('email'),
@@ -293,6 +311,32 @@ def normalize_users(users):
             enabled=user.get('enabled'),
         ) for user in users
     ]
+    return meta.obj_list_to_dict(ret)
+
+
+def normalize_domains(domains):
+    ret = [
+        dict(
+            id=domain.get('id'),
+            name=domain.get('name'),
+            description=domain.get('description'),
+            enabled=domain.get('enabled'),
+        ) for domain in domains
+    ]
+    return meta.obj_list_to_dict(ret)
+
+
+def normalize_groups(domains):
+    """Normalize Identity groups."""
+    ret = [
+        dict(
+            id=domain.get('id'),
+            name=domain.get('name'),
+            description=domain.get('description'),
+            domain_id=domain.get('domain_id'),
+        ) for domain in domains
+    ]
+    return meta.obj_list_to_dict(ret)
 
 
 def valid_kwargs(*valid_args):
@@ -319,3 +363,69 @@ def valid_kwargs(*valid_args):
                     "'{arg}'".format(f=inspect.stack()[1][3], arg=k))
         return func(*args, **kwargs)
     return func_wrapper
+
+
+def cache_on_arguments(*cache_on_args, **cache_on_kwargs):
+    def _inner_cache_on_arguments(func):
+        def _cache_decorator(obj, *args, **kwargs):
+            the_method = obj._cache.cache_on_arguments(
+                *cache_on_args, **cache_on_kwargs)(
+                    func.__get__(obj, type(obj)))
+            return the_method(*args, **kwargs)
+
+        def invalidate(obj, *args, **kwargs):
+            return obj._cache.cache_on_arguments()(func).invalidate(
+                *args, **kwargs)
+
+        _cache_decorator.invalidate = invalidate
+        _cache_decorator.func = func
+        _decorated_methods.append(func.__name__)
+
+        return _cache_decorator
+    return _inner_cache_on_arguments
+
+
+@contextlib.contextmanager
+def neutron_exceptions(error_message):
+    try:
+        yield
+    except neutron_exc.NotFound as e:
+        raise exc.OpenStackCloudResourceNotFound(
+            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
+    except neutron_exc.NeutronClientException as e:
+        if e.status_code == 404:
+            raise exc.OpenStackCloudURINotFound(
+                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
+        else:
+            raise exc.OpenStackCloudException(
+                "{msg}: {exc}".format(msg=error_message, exc=str(e)))
+    except Exception as e:
+        raise exc.OpenStackCloudException(
+            "{msg}: {exc}".format(msg=error_message, exc=str(e)))
+
+
+@contextlib.contextmanager
+def shade_exceptions(error_message=None):
+    """Context manager for dealing with shade exceptions.
+
+    :param string error_message: String to use for the exception message
+        content on non-OpenStackCloudExceptions.
+
+    Useful for avoiding wrapping shade OpenStackCloudException exceptions
+    within themselves. Code called from within the context may throw such
+    exceptions without having to catch and reraise them.
+
+    Non-OpenStackCloudException exceptions thrown within the context will
+    be wrapped and the exception message will be appended to the given error
+    message.
+    """
+    try:
+        yield
+    except exc.OpenStackCloudException:
+        raise
+    except Exception as e:
+        if error_message is not None:
+            message = "{msg}: {exc}".format(msg=error_message, exc=str(e))
+        else:
+            message = str(e)
+        raise exc.OpenStackCloudException(message)
