@@ -20,10 +20,12 @@ import abc
 import sys
 import threading
 import time
+import types
 
 import six
 
 from shade import _log
+from shade import meta
 
 
 @six.add_metaclass(abc.ABCMeta)
@@ -48,15 +50,20 @@ class Task(object):
         self._exception = None
         self._traceback = None
         self._result = None
+        self._response = None
         self._finished = threading.Event()
         self.args = kw
+        self.requests = False
 
     @abc.abstractmethod
     def main(self, client):
         """ Override this method with the actual workload to be performed """
 
     def done(self, result):
-        self._result = result
+        if self.requests:
+            self._response, self._result = result
+        else:
+            self._result = result
         self._finished.set()
 
     def exception(self, e, tb):
@@ -66,10 +73,20 @@ class Task(object):
 
     def wait(self):
         self._finished.wait()
+        # TODO(mordred): We store the raw requests response if there is
+        # one now. So we should probably do an error handler to throw
+        # some exceptions if it's not 200
         if self._exception:
             six.reraise(type(self._exception), self._exception,
                         self._traceback)
-        return self._result
+
+        if type(self._result) == list:
+            return meta.obj_list_to_dict(self._result)
+        elif type(self._result) not in (bool, int, float, str, set,
+                                        types.GeneratorType):
+            return meta.obj_to_dict(self._result)
+        else:
+            return self._result
 
     def run(self, client):
         try:
