@@ -95,10 +95,19 @@ class FakeUser(object):
 
 
 class FakeVolume(object):
-    def __init__(self, id, status, display_name):
+    def __init__(self, id, status, display_name, attachments=[]):
         self.id = id
         self.status = status
         self.display_name = display_name
+        self.attachments = attachments
+
+
+class FakeVolumeSnapshot(object):
+    def __init__(self, id, status, display_name, display_description):
+        self.id = id
+        self.status = status
+        self.display_name = display_name
+        self.display_description = display_description
 
 
 class FakeMachine(object):
@@ -161,3 +170,11 @@ class FakeRole(object):
     def __init__(self, id, name):
         self.id = id
         self.name = name
+
+
+class FakeGroup(object):
+    def __init__(self, id, name, description, domain=None):
+        self.id = id
+        self.name = name
+        self.description = description
+        self.domain = domain
