@@ -19,6 +19,7 @@ import testtools
 import yaml
 
 import shade
+import shade.openstackcloud
 from shade import exc
 from shade import meta
 from shade.tests import fakes
@@ -32,6 +33,9 @@ class TestMemoryCache(base.TestCase):
         {
             'max_age': 90,
             'class': 'dogpile.cache.memory',
+            'expiration': {
+                'server': 1,
+            },
         },
         'clouds':
         {
@@ -290,13 +294,13 @@ class TestMemoryCache(base.TestCase):
             # therefore we should _not_ expect to see the new one here
             self.assertEqual([first_image], self.cloud.list_images())
 
-    def _call_create_image(self, name, container=None):
+    def _call_create_image(self, name, container=None, **kwargs):
         imagefile = tempfile.NamedTemporaryFile(delete=False)
         imagefile.write(b'\0')
         imagefile.close()
         self.cloud.create_image(
             name, imagefile.name, container=container, wait=True,
-            is_public=False)
+            is_public=False, **kwargs)
 
     @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
     @mock.patch.object(shade.OpenStackCloud, 'glance_client')
@@ -314,10 +318,10 @@ class TestMemoryCache(base.TestCase):
                 'properties': {'owner_specified.shade.md5': mock.ANY,
                                'owner_specified.shade.sha256': mock.ANY,
                                'is_public': False}}
-        glance_mock.images.create.assert_called_with(**args)
-        glance_mock.images.update.assert_called_with(data=mock.ANY,
-                                                     image=fake_image)
         fake_image_dict = meta.obj_to_dict(fake_image)
+        glance_mock.images.create.assert_called_with(**args)
+        glance_mock.images.update.assert_called_with(
+            data=mock.ANY, image=fake_image_dict)
         self.assertEqual([fake_image_dict], self.cloud.list_images())
 
     @mock.patch.object(occ.cloud_config.CloudConfig, 'get_api_version')
@@ -332,12 +336,13 @@ class TestMemoryCache(base.TestCase):
         fake_image = fakes.FakeImage('42', '42 name', 'success')
         glance_mock.images.create.return_value = fake_image
         glance_mock.images.list.return_value = [fake_image]
-        self._call_create_image('42 name')
+        self._call_create_image('42 name', min_disk=0, min_ram=0)
         args = {'name': '42 name',
                 'container_format': 'bare', 'disk_format': 'qcow2',
                 'owner_specified.shade.md5': mock.ANY,
                 'owner_specified.shade.sha256': mock.ANY,
-                'visibility': 'private'}
+                'visibility': 'private',
+                'min_disk': 0, 'min_ram': 0}
         glance_mock.images.create.assert_called_with(**args)
         glance_mock.images.upload.assert_called_with(
             image_data=mock.ANY, image_id=fake_image.id)
@@ -386,8 +391,8 @@ class TestMemoryCache(base.TestCase):
         fake_image.update({
             'id': '99',
             'name': '99 name',
-            shade.IMAGE_MD5_KEY: fake_md5,
-            shade.IMAGE_SHA256_KEY: fake_sha256,
+            shade.openstackcloud.IMAGE_MD5_KEY: fake_md5,
+            shade.openstackcloud.IMAGE_SHA256_KEY: fake_sha256,
         })
         glance_mock.images.list.return_value = [fake_image]
 
