@@ -17,12 +17,12 @@ import munch
 
 import glanceclient
 from heatclient import client as heat_client
-from keystoneclient.v2_0 import client as k2_client
-from keystoneclient.v3 import client as k3_client
 from neutronclient.common import exceptions as n_exc
+import testtools
 
-import os_client_config.cloud_config
+from os_client_config import cloud_config
 import shade
+from shade import _utils
 from shade import exc
 from shade import meta
 from shade.tests.unit import base
@@ -37,33 +37,6 @@ class TestShade(base.TestCase):
     def test_openstack_cloud(self):
         self.assertIsInstance(self.cloud, shade.OpenStackCloud)
 
-    @mock.patch.object(
-        os_client_config.cloud_config.CloudConfig, 'get_api_version')
-    def test_get_client_v2(self, mock_api_version):
-        mock_api_version.return_value = '2'
-
-        self.assertIs(
-            self.cloud._get_identity_client_class(),
-            k2_client.Client)
-
-    @mock.patch.object(
-        os_client_config.cloud_config.CloudConfig, 'get_api_version')
-    def test_get_client_v3(self, mock_api_version):
-        mock_api_version.return_value = '3'
-
-        self.assertIs(
-            self.cloud._get_identity_client_class(),
-            k3_client.Client)
-
-    @mock.patch.object(
-        os_client_config.cloud_config.CloudConfig, 'get_api_version')
-    def test_get_client_v4(self, mock_api_version):
-        mock_api_version.return_value = '4'
-
-        self.assertRaises(
-            exc.OpenStackCloudException,
-            self.cloud._get_identity_client_class)
-
     @mock.patch.object(shade.OpenStackCloud, 'search_images')
     def test_get_images(self, mock_search):
         image1 = dict(id='123', name='mickey')
@@ -98,33 +71,35 @@ class TestShade(base.TestCase):
         self.assertRaises(exc.OpenStackCloudException,
                           self.cloud.list_servers)
 
-    @mock.patch.object(shade.OpenStackCloud, 'get_session_endpoint')
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
     @mock.patch.object(glanceclient, 'Client')
     def test_glance_args(
-            self, mock_client, mock_keystone_session, mock_endpoint):
-        mock_keystone_session.return_value = None
-        mock_endpoint.return_value = 'http://example.com/v2'
+            self, mock_client, get_session_mock):
+        session_mock = mock.Mock()
+        session_mock.get_endpoint.return_value = 'http://example.com/v2'
+        get_session_mock.return_value = session_mock
         self.cloud.glance_client
         mock_client.assert_called_with(
+            '2',
             endpoint='http://example.com',
-            version='2', region_name='', service_name=None,
+            region_name='', service_name=None,
             interface='public',
             service_type='image', session=mock.ANY,
         )
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
     @mock.patch.object(heat_client, 'Client')
-    def test_heat_args(self, mock_client, mock_keystone_session):
-        mock_keystone_session.return_value = None
+    def test_heat_args(self, mock_client, get_session_mock):
+        session_mock = mock.Mock()
+        get_session_mock.return_value = session_mock
         self.cloud.heat_client
         mock_client.assert_called_with(
+            '1',
             endpoint_type='public',
             region_name='',
             service_name=None,
             service_type='orchestration',
             session=mock.ANY,
-            version='1'
         )
 
     @mock.patch.object(shade.OpenStackCloud, 'search_subnets')
@@ -384,22 +359,6 @@ class TestShade(base.TestCase):
             self.assertRaises(exc.OpenStackCloudException,
                               self.cloud.list_networks)
 
-    @mock.patch.object(shade.OpenStackCloud, 'list_zones')
-    def test_get_zone(self, mock_search):
-        zone1 = dict(id='123', name='mickey')
-        mock_search.return_value = [zone1]
-        r = self.cloud.get_zone('mickey')
-        self.assertIsNotNone(r)
-        self.assertDictEqual(zone1, r)
-
-    @mock.patch.object(shade.OpenStackCloud, '_list_records')
-    def test_get_record(self, mock_search):
-        record1 = dict(id='123', name='mickey', domain_id='mickey.domain')
-        mock_search.return_value = [record1]
-        r = self.cloud._get_record('mickey.domain', 'mickey')
-        self.assertIsNotNone(r)
-        self.assertDictEqual(record1, r)
-
     @mock.patch.object(shade._tasks.ServerList, 'main')
     def test_list_servers(self, mock_serverlist):
         '''This test verifies that calling list_servers results in a call
@@ -433,3 +392,37 @@ class TestShade(base.TestCase):
         self.assertEquals(len(r), mock_get_hostvars_from_server.call_count)
         self.assertEquals('server1', r[0]['name'])
         self.assertEquals('server2', r[1]['name'])
+
+    def test_iterate_timeout_bad_wait(self):
+        with testtools.ExpectedException(
+                exc.OpenStackCloudException,
+                "Wait value must be an int or float value."):
+            for count in _utils._iterate_timeout(
+                    1, "test_iterate_timeout_bad_wait", wait="timeishard"):
+                pass
+
+    @mock.patch('time.sleep')
+    def test_iterate_timeout_str_wait(self, mock_sleep):
+        iter = _utils._iterate_timeout(
+            10, "test_iterate_timeout_str_wait", wait="1.6")
+        next(iter)
+        next(iter)
+        mock_sleep.assert_called_with(1.6)
+
+    @mock.patch('time.sleep')
+    def test_iterate_timeout_int_wait(self, mock_sleep):
+        iter = _utils._iterate_timeout(
+            10, "test_iterate_timeout_int_wait", wait=1)
+        next(iter)
+        next(iter)
+        mock_sleep.assert_called_with(1.0)
+
+    @mock.patch('time.sleep')
+    def test_iterate_timeout_timeout(self, mock_sleep):
+        message = "timeout test"
+        with testtools.ExpectedException(
+                exc.OpenStackCloudTimeout,
+                message):
+            for count in _utils._iterate_timeout(0.1, message, wait=1):
+                pass
+        mock_sleep.assert_called_with(1.0)
