@@ -12,13 +12,14 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-from keystoneauth1 import plugin as ksc_plugin
+from keystoneauth1 import plugin as ksa_plugin
 
 import mock
 import testtools
 
-import os_client_config.cloud_config
+from os_client_config import cloud_config
 import shade
+import munch
 from shade import exc
 from shade import meta
 from shade.tests import fakes
@@ -363,6 +364,180 @@ class TestShadeOperator(base.TestCase):
             '00000000-0000-0000-0000-000000000000',
             expected_patch)
 
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_fail_active(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class active_machine:
+            uuid = machine_uuid
+            provision_state = "active"
+
+        mock_client.node.get.return_value = active_machine
+        self.assertRaises(
+            shade.OpenStackCloudException,
+            self.cloud.inspect_machine,
+            machine_uuid,
+            wait=True,
+            timeout=1)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_failed(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class inspect_failed_machine:
+            uuid = machine_uuid
+            provision_state = "inspect failed"
+            last_error = "kaboom"
+
+        mock_client.node.get.return_value = inspect_failed_machine
+        self.cloud.inspect_machine(machine_uuid)
+        self.assertTrue(mock_client.node.set_provision_state.called)
+        self.assertEqual(
+            mock_client.node.set_provision_state.call_count, 1)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_managable(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class manageable_machine:
+            uuid = machine_uuid
+            provision_state = "manageable"
+
+        mock_client.node.get.return_value = manageable_machine
+        self.cloud.inspect_machine(machine_uuid)
+        self.assertEqual(
+            mock_client.node.set_provision_state.call_count, 1)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_available(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class available_machine:
+            uuid = machine_uuid
+            provision_state = "available"
+
+        class manageable_machine:
+            uuid = machine_uuid
+            provision_state = "manageable"
+
+        class inspecting_machine:
+            uuid = machine_uuid
+            provision_state = "inspecting"
+
+        mock_client.node.get.side_effect = iter([
+            available_machine,
+            available_machine,
+            manageable_machine,
+            manageable_machine,
+            inspecting_machine])
+        self.cloud.inspect_machine(machine_uuid)
+        self.assertTrue(mock_client.node.set_provision_state.called)
+        self.assertEqual(
+            mock_client.node.set_provision_state.call_count, 3)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_available_wait(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class available_machine:
+            uuid = machine_uuid
+            provision_state = "available"
+
+        class manageable_machine:
+            uuid = machine_uuid
+            provision_state = "manageable"
+
+        class inspecting_machine:
+            uuid = machine_uuid
+            provision_state = "inspecting"
+
+        mock_client.node.get.side_effect = iter([
+            available_machine,
+            available_machine,
+            manageable_machine,
+            inspecting_machine,
+            manageable_machine,
+            available_machine,
+            available_machine])
+        expected_return_value = dict(
+            uuid=machine_uuid,
+            provision_state="available"
+        )
+
+        return_value = self.cloud.inspect_machine(
+            machine_uuid, wait=True, timeout=1)
+        self.assertTrue(mock_client.node.set_provision_state.called)
+        self.assertEqual(
+            mock_client.node.set_provision_state.call_count, 3)
+        self.assertDictEqual(expected_return_value, return_value)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_wait(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class manageable_machine:
+            uuid = machine_uuid
+            provision_state = "manageable"
+
+        class inspecting_machine:
+            uuid = machine_uuid
+            provision_state = "inspecting"
+
+        expected_return_value = dict(
+            uuid=machine_uuid,
+            provision_state="manageable"
+        )
+        mock_client.node.get.side_effect = iter([
+            manageable_machine,
+            inspecting_machine,
+            inspecting_machine,
+            manageable_machine,
+            manageable_machine])
+
+        return_value = self.cloud.inspect_machine(
+            machine_uuid, wait=True, timeout=1)
+        self.assertDictEqual(expected_return_value, return_value)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_inspect_machine_inspect_failed(self, mock_client):
+
+        machine_uuid = '00000000-0000-0000-0000-000000000000'
+
+        class manageable_machine:
+            uuid = machine_uuid
+            provision_state = "manageable"
+            last_error = None
+
+        class inspecting_machine:
+            uuid = machine_uuid
+            provision_state = "inspecting"
+            last_error = None
+
+        class inspect_failed_machine:
+            uuid = machine_uuid
+            provision_state = "inspect failed"
+            last_error = "kaboom"
+
+        mock_client.node.get.side_effect = iter([
+            manageable_machine,
+            inspecting_machine,
+            inspect_failed_machine])
+        self.assertRaises(
+            shade.OpenStackCloudException,
+            self.cloud.inspect_machine,
+            machine_uuid,
+            wait=True,
+            timeout=1)
+        self.assertEqual(
+            mock_client.node.set_provision_state.call_count, 1)
+        self.assertEqual(mock_client.node.get.call_count, 3)
+
     @mock.patch.object(shade.OperatorCloud, 'ironic_client')
     def test_register_machine(self, mock_client):
         class fake_node:
@@ -660,17 +835,95 @@ class TestShadeOperator(base.TestCase):
 
     @mock.patch.object(shade.OperatorCloud, 'ironic_client')
     def test_node_set_provision_state(self, mock_client):
+
+        class active_node_state:
+            provision_state = "active"
+
+        active_return_value = dict(
+            provision_state="active")
+
         mock_client.node.set_provision_state.return_value = None
+        mock_client.node.get.return_value = active_node_state
         node_id = 'node01'
         return_value = self.cloud.node_set_provision_state(
             node_id,
             'active',
             configdrive='http://127.0.0.1/file.iso')
-        self.assertEqual({}, return_value)
+        self.assertEqual(active_return_value, return_value)
         mock_client.node.set_provision_state.assert_called_with(
             node_uuid='node01',
             state='active',
             configdrive='http://127.0.0.1/file.iso')
+        self.assertTrue(mock_client.node.get.called)
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_node_set_provision_state_wait_timeout(self, mock_client):
+        class deploying_node_state:
+            provision_state = "deploying"
+
+        class active_node_state:
+            provision_state = "active"
+
+        class managable_node_state:
+            provision_state = "managable"
+
+        class available_node_state:
+            provision_state = "available"
+
+        active_return_value = dict(
+            provision_state="active")
+        mock_client.node.get.return_value = active_node_state
+        mock_client.node.set_provision_state.return_value = None
+        node_id = 'node01'
+        return_value = self.cloud.node_set_provision_state(
+            node_id,
+            'active',
+            configdrive='http://127.0.0.1/file.iso',
+            wait=True)
+
+        self.assertEqual(active_return_value, return_value)
+        mock_client.node.set_provision_state.assert_called_with(
+            node_uuid='node01',
+            state='active',
+            configdrive='http://127.0.0.1/file.iso')
+        self.assertTrue(mock_client.node.get.called)
+        mock_client.mock_reset()
+        mock_client.node.get.return_value = deploying_node_state
+        self.assertRaises(
+            shade.OpenStackCloudException,
+            self.cloud.node_set_provision_state,
+            node_id,
+            'active',
+            configdrive='http://127.0.0.1/file.iso',
+            wait=True,
+            timeout=0.001)
+        self.assertTrue(mock_client.node.get.called)
+        mock_client.node.set_provision_state.assert_called_with(
+            node_uuid='node01',
+            state='active',
+            configdrive='http://127.0.0.1/file.iso')
+
+    @mock.patch.object(shade.OperatorCloud, 'ironic_client')
+    def test_node_set_provision_state_wait_provide(self, mock_client):
+
+        class managable_node_state:
+            provision_state = "managable"
+
+        class available_node_state:
+            provision_state = "available"
+
+        node_provide_return_value = dict(
+            provision_state="available")
+
+        mock_client.node.get.side_effect = iter([
+            managable_node_state,
+            available_node_state])
+        return_value = self.cloud.node_set_provision_state(
+            'test_node',
+            'provide',
+            wait=True)
+        self.assertEqual(mock_client.node.get.call_count, 2)
+        self.assertDictEqual(node_provide_return_value, return_value)
 
     @mock.patch.object(shade.OperatorCloud, 'ironic_client')
     def test_activate_node(self, mock_client):
@@ -739,50 +992,79 @@ class TestShadeOperator(base.TestCase):
         self.assertEqual('22', self.cloud.get_image_id('22'))
         self.assertEqual('22', self.cloud.get_image_id('22 name'))
 
-    @mock.patch.object(
-        os_client_config.cloud_config.CloudConfig, 'get_endpoint')
+    @mock.patch.object(cloud_config.CloudConfig, 'get_endpoint')
     def test_get_session_endpoint_provided(self, fake_get_endpoint):
         fake_get_endpoint.return_value = 'http://fake.url'
         self.assertEqual(
             'http://fake.url', self.cloud.get_session_endpoint('image'))
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
-    def test_get_session_endpoint_session(self, session_mock):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_get_session_endpoint_session(self, get_session_mock):
+        session_mock = mock.Mock()
         session_mock.get_endpoint.return_value = 'http://fake.url'
+        get_session_mock.return_value = session_mock
         self.assertEqual(
             'http://fake.url', self.cloud.get_session_endpoint('image'))
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
-    def test_get_session_endpoint_exception(self, session_mock):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_get_session_endpoint_exception(self, get_session_mock):
         class FakeException(Exception):
             pass
 
         def side_effect(*args, **kwargs):
             raise FakeException("No service")
+        session_mock = mock.Mock()
         session_mock.get_endpoint.side_effect = side_effect
+        get_session_mock.return_value = session_mock
+        self.cloud.name = 'testcloud'
+        self.cloud.region_name = 'testregion'
         with testtools.ExpectedException(
                 exc.OpenStackCloudException,
-                "Error getting image endpoint: No service"):
+                "Error getting image endpoint on testcloud:testregion:"
+                " No service"):
             self.cloud.get_session_endpoint("image")
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
-    def test_get_session_endpoint_unavailable(self, session_mock):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_get_session_endpoint_unavailable(self, get_session_mock):
+        session_mock = mock.Mock()
         session_mock.get_endpoint.return_value = None
+        get_session_mock.return_value = session_mock
         image_endpoint = self.cloud.get_session_endpoint("image")
         self.assertIsNone(image_endpoint)
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
-    def test_get_session_endpoint_identity(self, session_mock):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_get_session_endpoint_identity(self, get_session_mock):
+        session_mock = mock.Mock()
+        get_session_mock.return_value = session_mock
         self.cloud.get_session_endpoint('identity')
         session_mock.get_endpoint.assert_called_with(
-            interface=ksc_plugin.AUTH_INTERFACE)
+            interface=ksa_plugin.AUTH_INTERFACE)
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
-    def test_has_service_no(self, session_mock):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_has_service_no(self, get_session_mock):
+        session_mock = mock.Mock()
         session_mock.get_endpoint.return_value = None
+        get_session_mock.return_value = session_mock
         self.assertFalse(self.cloud.has_service("image"))
 
-    @mock.patch.object(shade.OpenStackCloud, 'keystone_session')
-    def test_has_service_yes(self, session_mock):
+    @mock.patch.object(cloud_config.CloudConfig, 'get_session')
+    def test_has_service_yes(self, get_session_mock):
+        session_mock = mock.Mock()
         session_mock.get_endpoint.return_value = 'http://fake.url'
+        get_session_mock.return_value = session_mock
         self.assertTrue(self.cloud.has_service("image"))
+
+    @mock.patch.object(shade._tasks.HypervisorList, 'main')
+    def test_list_hypervisors(self, mock_hypervisorlist):
+        '''This test verifies that calling list_hypervisors results in a call
+        to the HypervisorList task.'''
+        mock_hypervisorlist.return_value = [
+            munch.Munch({'hypervisor_hostname': 'testserver1',
+                         'id': '1'}),
+            munch.Munch({'hypervisor_hostname': 'testserver2',
+                         'id': '2'})
+        ]
+
+        r = self.cloud.list_hypervisors()
+        self.assertEquals(2, len(r))
+        self.assertEquals('testserver1', r[0]['hypervisor_hostname'])
